from jinja2 import Template

class TerraformTemplate(Template):
    pass



standard_resource = Template(
'''
resource "{{ object_name }}" {
{% for option, value in object_options.items() -%}
{{ option | indent }} = {{ value[1:] if value.startswith('_') else (value | quote) }}
{% endfor -%}
}
'''
)



from jinja2.filters import FILTERS, environmentfilter

@environmentfilter
def filter_quote(environment, value):
    return f'"{value}"'

@environmentfilter
def filter_indent(environment, value, size=2):
    indent = " " * size
    return f'{indent} {value}'


FILTERS["quote"] = filter_quote
FILTERS["indent"] = filter_indent



from platformer.templates.base_terraform import standard_resource

class TerraformCoder:
    def __init__(self, pf_definition):
        self.pf_definition = pf_definition
        self.modules = [module for module in pf_definition["modules"]]

    def render(self, resource):
        return standard_resource.render(
            object_name = resource["name"],
            object_options = resource["options"]
        )

    def filter_module_type(self, query):
        return list(filter(
            lambda mod: mod["type"] == query, self.modules
        ))

    def get_rendered_list(self):
        return [
            self.render(module)
            for module in self.filter_module_type("raw_resource")
        ]

    def print_code(self):
        return '\n'.join(self.get_rendered_list())





import click
from pprint import pprint
from platformer.core.parser import PlatformerConfigReader

@click.command()
@click.option('-f', '--file', type=click.File('r'), required=True)
def oneclick(file):
    """
    | Template based code generation and deployment
    """
    pass



import click

@click.group()
def deploy():
    """| Git repository and pipeline configuration""" 
    pass

@deploy.command()
def azdo():
    """| Azure Devops Deployer"""
    print("Azure Devops module")

@deploy.command()
def github():
    """| Github Deployer"""
    print("Github module")



import click
from platformer.core.parser import PlatformerConfigReader
from platformer.core.coders.terraform import TerraformCoder

@click.group()
def code():
    """| Code generator for terraform, ansible or yaml pipelines"""
    pass

@code.command()
@click.option('-f', '--file', type=click.File('r'), required=False)
def terraform(file):
    """
    | Terraform code generator
    """
    if file:
        config = PlatformerConfigReader(file.read())
        terraform_code = TerraformCoder(config.get_terraform())
        click.echo(terraform_code.print_code())
    else: 
        click.echo("No input files")

@code.command()
def ansible():
    """
    | Ansible Playbook generator
    """
    print("Ansible module")

@code.command()
def pipeline():
    """
    | YAML Pipeline generator (Github, Azure Devops)
    """
    print("Pipeline module")



import yaml

class PlatformerConfigReader():
    def __init__(self, yaml_text):
        self.config_yaml = yaml.safe_load(yaml_text)
        self.config =  self._input_defaults(self.config_yaml)

    def _find_definition(self, name):
        not_found = f"Definition {name} not found on config file"
        if result := self.config.get(name, None):
            return result
        else:
            return not_found
            exit(-1)


    def _input_defaults(self, config):
        config_merged = config
        print("Fill default values")
        return config_merged

    def get_terraform(self, ):
        return self._find_definition("terraform")









import click

@click.group()
def test():
    """| Test code running on local machine""" 
    pass

@test.command()
def terraform():
    """| Run Generated Terraform code"""
    print("Run Terraform Init")
    print("Run Terraform Plan")
    print("Run Terraform Apply")

@test.command()
def ansible():
    """| Run Generated Ansible code"""
    print("Run Ansible Playbook")



import click
import collections
from platformer.cli.code import code
from platformer.cli.test import test
from platformer.cli.deploy import deploy
from platformer.cli.oneclick import oneclick
from platformer import __version__


class OrderedGroup(click.Group):
    def __init__(self, name=None, commands=None, **attrs):
        super(OrderedGroup, self).__init__(name, commands, **attrs)
        #: the registered subcommands by their exported names.
        self.commands = commands or collections.OrderedDict()

    def list_commands(self, ctx):
        return self.commands


@click.group(cls=OrderedGroup)
@click.version_option(
    prog_name = 'Platformer',
    version   = __version__
)
def cli():
    """
    Platform Builder Framework

    \b
    :: Your platform based on git code
    :: Read the docs: https://github.com/rfsantanna/platformer
    \b
    Features:
      - Terraform HCL Code Generator
      - YAML Code Generator (Pipelines, Ansible)
      - Git Remote Repository Management
      - Infrastructure based Testing
    """
    ...

cli.add_command(code)
cli.add_command(test)
cli.add_command(deploy)
cli.add_command(oneclick)



import yaml

class PlatformerConfigReader():
    def __init__(self, yaml_text):
        self.config_yaml = yaml.safe_load(yaml_text)
        self.config =  self.input_defaults(self.config_yaml)

    def input_defaults(self, config):
        config_merged = config
        print("Fill default values")
        return config_merged





